class _Node:
    __slots__ = '_element','_next'
    def __init__(self,element,next):
        self._element = element
        self._next = next
    
class Stack:
    def __init__(self,N = 'inf'):
        self._top = None 
        self._N = N
        self._size = 0
    
    # Size of linkedlist
    def __len__(self):
        return self._size
    # Print stack using print function
    def __str__(self):
        if self.isEmpty():
            return "Stack is Empty!"
        s = ""
        p = self._top
        while p:
            s = str(p._element) + " "+s
            p = p._next
        return s
    # Check if list is empty
    def isEmpty(self):
        return self._size == 0
    # Check if stack is full
    def isFull(self):
        return self._size == self._N

    # Push an element to the stack
    def push(self,e):
        if self.isFull():
            print("Stack is Full!")
            return
        newest = _Node(e,None)
        if self.isEmpty():
            self._top = newest
        else:
            newest._next = self._top
            self._top = newest
        self._size += 1

    # Pop an element out of the stack
    def pop(self):
        if self.isEmpty():
            print("Stack is Empty!")
            return
        elif self._size ==1:
            e = self._top._element
            self.clear()
            return e
        else:
            e = self._top._element
            self._top = self._top._next
            self._size -= 1
            return e
        
    # Return top element of the stack
    def top(self):
        if self.isEmpty():
            print("Stack is Empty!")
            return
        else:
            return self._top._element

    # Display the Stack
    def display(self):
        if self.isEmpty():
            return "Stack is Empty!"
        s = ""
        p = self._top
        while p:
            s = "| "+str(p._element)+" " +s
            p = p._next
        print(s)
    
    # Delete the stack
    def clear(self):
        self._size = 0
        self._top = None

if __name__ == "__main__":
    s = Stack(5)
    s.push(1)
    s.push(4)
    s.push(2)
    s.push(8)
    s.push(6)
    print(len(s))
    print(s.isEmpty())
    print(s.isFull())
    s.push(5)
    print(s.pop())
    print(s.top())
    s.display()
    print(s)
    s.clear()
    print(s)


def search(arr,key):
    left = 0
    right = len(arr)-1
    while left<=right:
        mid = (left+right)//2
        if arr[mid]==key:
            return mid
        elif arr[mid]>key:
            right = mid -1
        elif arr[mid]<key:
            left = mid +1 
    return -1

if __name__ == "__main__":
    a = eval(input('Array: '))
    a.sort()
    k = int(input('Search for: '))
    print("Found at:",search(a,k))


def merge(s1,s2,s,mode):
    i = j = 0
    if mode==0:
        while i+j<len(s):
            if j==len(s2) or (i<len(s1) and s1[i]<s2[j]):
                s[i+j]=s1[i]
                i += 1
            else:
                s[i+j]=s2[j]
                j += 1
    elif mode==1:
        while i+j<len(s):
            if j==len(s2) or (i<len(s1) and s1[i]>s2[j]):
                s[i+j]=s1[i]
                i += 1
            else:
                s[i+j]=s2[j]
                j += 1        
def sort(s,mode=0):
    n = len(s)
    if n<2:
        return
    mid = n//2
    s1 = s[0:mid]
    s2 = s[mid:n]
    sort(s1,mode)
    sort(s2,mode)
    merge(s1,s2,s,mode)
    return s


if __name__=='__main__':
    import time
    seconds_1 = time.time()
    a = [9,6,5,8,7,2,3,0,1,4]
    print(sort(a))
    print(sort(a,1))
    seconds = time.time()
    print("runtime:",seconds-seconds_1)
    
'''
    i,j,k = left,mid+1,left  
    B = [0]*(right+1)
    while i<=mid and j<=right:
        if mode==0:                                 # Ascending
            if A[i]<A[j]:
                B[k] = A[i]
                i += 1
            else:
                B[k] = A[j] 
                j += 1
            k += 1 
        elif mode==1:                               # Descending 
            if A[i]>A[j]:
                B[k] = A[i]
                i += 1
            else:
                B[k] = A[j] 
                j += 1
            k += 1
    while i<=mid:
        B[k] = A[i]       
        i += 1
        k += 1
    while j<=right:
        B[k] = A[j]
        j += 1
        k += 1
    for m in range(left,right+1):
        A[m] = B[m]
def sort(arr,left,right,mode=0):
    if left<right:
        mid = (left+right)//2
        sort(arr,left,mid,mode)
        sort(arr,mid+1,right,mode)
        merge(arr,left,mid,right,mode)
    return arr'''


def sort(a,mode=0):
    if mode == 0:                                               # Ascending
        n = len(a)
        mx = max(a)
        cArray = [0]*(mx+1)
        for i in range(n):
            cArray[a[i]] += 1
        i,j = 0,0
        while i<mx+1:
            if cArray[i]>0:
                a[j] = i
                j += 1
                cArray[i] -= 1
            else:
                i += 1
        return a     
    elif mode == 1:                                             # Descending
        n = len(a)
        mx = max(a)
        cArray = [0]*(mx+1)
        for i in range(n):
            cArray[a[i]] += 1
        i = mx
        j = 0
        while i>=0:
            if cArray[i]>0:
                a[j] = i
                j += 1
                cArray[i] -= 1
            else:
                i -= 1
        return a

if __name__=='__main__':
    import time
    seconds_1 = time.time()
    a = [9,6,5,8,7,2,3,0,1,4]
    print(sort(a))
    print(sort(a,1))
    seconds = time.time()
    print("runtime:",seconds-seconds_1)


class Queue:
    def __init__(self,N = 'inf'):
        self._data = [] 
        self._N = N  

    # Size of Queue
    def __len__(self):
        return len(self._data)
    # Print Queue using print function
    def __str__(self):
        if self.isEmpty():
            return "Queue is Empty!"
        s = ""
        for i in self._data:
            s += str(i) + " "
        return s
    # Check if Queue is empty
    def isEmpty(self):
        return len(self._data) == 0
    # Check if Queue is full
    def isFull(self):
        return len(self._data) == self._N

    # Insert an element to the Queue
    def enqueue(self,e):
        if self.isFull():
            print("Queue is Full!")
            return
        else:
            self._data.append(e)

    # Remove an element out of the Queue
    def dequeue(self):
        if self.isEmpty():
            print("Queue is Empty!")
            return
        else:
            return self._data.pop(0)

    # Return first element of the Queue
    def first(self):
        if self.isEmpty():
            print("Queue is Empty!")
            return
        else:
            return self._data[0]
    
    # Display the Queue
    def display(self):
        for i in self._data:
            print(i,"|",end=" ")
        print()
    
    # Delete the Queue
    def clear(self):
        self._data = []
class _Node:
    __slots__='_element','_left','_right'

    def __init__(self,element,left=None,right=None):
        self._element = element
        self._left = left
        self._right = right

class Tree:
    def __init__(self):
        self._root = None

    def makeTree(self,e,left,right):
        self._root = _Node(e,left._root,right._root)

    def inOrder(self,tRoot):
        if tRoot:
            self.inOrder(tRoot._left)
            print(tRoot._element,end=" ")
            self.inOrder(tRoot._right)
    def preOrder(self,tRoot):
        if tRoot:
            print(tRoot._element,end=" ")
            self.preOrder(tRoot._left)
            self.preOrder(tRoot._right)
    def postOrder(self,tRoot):
        if tRoot:
            self.postOrder(tRoot._left)
            self.postOrder(tRoot._right)
            print(tRoot._element,end=" ")
    def levelOrder(self):
        q = Queue()
        t = self._root
        print(t._element,end=" ")
        q.enqueue(t)
        while not q.isEmpty():
            t = q.dequeue()
            if t._left:
                print(t._left._element,end=" ")
                q.enqueue(t._left)
            if t._right:
                print(t._right._element,end=" ")
                q.enqueue(t._right)
        
    def count(self,tRoot):
        if tRoot:
            x = self.count(tRoot._left)   
            y = self.count(tRoot._right)   
            return x+y+1
        return 0

    def calHeight(self,tRoot):
        if tRoot:
            x = self.calHeight(tRoot._left)   
            y = self.calHeight(tRoot._right)   
            if x > y:
                return x+1
            else:
                return y+1
        return 0

    def height(self,tRoot):
        h = self.calHeight(tRoot)
        return h-1
if __name__ == '__main__':
    x = Tree()
    y = Tree()
    z = Tree()
    r = Tree()
    s = Tree()
    t = Tree()
    a = Tree()
    x.makeTree(40,a,a)
    y.makeTree(60,a,a)
    z.makeTree(20,x,a)
    r.makeTree(50,a,y)
    s.makeTree(30,r,a)
    t.makeTree(10,z,s)
    print("In order:")
    t.inOrder(t._root)
    print()
    print("Pre order:")
    t.preOrder(t._root)
    print()
    print("Post order:")
    t.postOrder(t._root)
    print()
    print("Post order:")
    t.levelOrder()
    print()
    print("No of Nodes:",end=" ")
    print(t.count(t._root))
    print("Height of tree:",end=" ")
    print(t.height(t._root))


def sort(arr,mode=0):
    if mode==0:                     # Ascending Order
        l = len(arr)
        for i in range (0,l-1):
            s = i
            for j in range(i+1,l):
                if arr[j]<arr[s]:
                    s = j
            arr[i],arr[s] = arr[s],arr[i]
        return arr
    elif mode==1:                   # Descending Order
        l = len(arr)
        for i in range (0,l-1):
            s = i
            for j in range(i+1,l):
                if arr[j]>arr[s]:
                    s = j
            arr[i],arr[s] = arr[s],arr[i]
        return arr
        
if __name__=='__main__':
    import time
    seconds_1 = time.time()
    a = [9,6,5,8,7,2,3,0,1,4]
    print(sort(a))
    print(sort(a,1))
    seconds = time.time()
    print("runtime:",seconds-seconds_1)


def sort(a,low,high,mode=0):
    if low<high:
        p = partition(a,low,high,mode)
        sort(a,low,p-1,mode)
        sort(a,p+1,high,mode)
    return a

def partition(a,low,high,mode):
    pivot = a[low]
    i,j = low+1,high
    if mode == 0:                                               # Ascending
        while True:
            while i<=j and a[i]<=pivot:
                i += 1
            while i<=j and a[j]>=pivot:
                j -= 1
            if i<=j:
                a[i],a[j] = a[j],a[i]
            else:
                break
        a[low],a[j] = a[j],a[low]
        return j  
    elif mode == 1:                                             # Descending
        while True:
            while i<=j and a[i]>=pivot:
                i += 1
            while i<=j and a[j]<=pivot:
                j -= 1
            if i<=j:
                a[i],a[j] = a[j],a[i]
            else:
                break
        a[low],a[j] = a[j],a[low]
        return j 

if __name__=='__main__':
    import time
    seconds_1 = time.time()
    a = [9,6,5,8,7,2,3,0,1,4]
    print(sort(a,0,len(a)-1))
    print(sort(a,0,len(a)-1,1))
    seconds = time.time()
    print("runtime:",seconds-seconds_1)


def search(arr,key):
    length = len(arr)
    index = 0
    while index<length:
        if arr[index]==key:
            return index 
        index += 1
    return -1

if __name__ == "__main__":
    a = eval(input('Array: '))
    k = int(input('Search for: '))
    print("Found at:",search(a,k))


from setuptools import setup, find_packages

setup(
    name= 'dimppy', 
    version='0.0.0.3',
    author='Armaan Chauhan',
    author_email='armaanchauhan268@gmail.com',
    description='Data Structures and Algorithms implementation in Python', 
    long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/Armaan-268/DSA_in_Python',
    license="MIT",
    classifiers=[
    'Development Status :: 3 - Alpha', 
    'Intended Audience :: Education',
    'Topic :: Software Development :: Libraries',
    'Operating System :: OS Independent',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
    ], 
    keywords='datastructures,algorithms',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['']
)


def search(arr,key,left,right):            
    if left>right:
        return -1
    else:
        mid = (left+right)//2
        if arr[mid]==key:
            return mid
        elif arr[mid]<key:
            return search(arr,key,mid+1,right)
        elif arr[mid]>key:
            return search(arr,key,left,mid-1)

if __name__ == "__main__":
    a = eval(input('Array: '))
    a.sort()
    l = len(a)
    k = int(input('Search for: '))
    print("Found at:",search(a,k,0,l-1))